package main

import (
	commonConfig "cicv-data-closedloop/aarch64/jili/common/config"
	commonService "cicv-data-closedloop/aarch64/jili/common/service"
	"cicv-data-closedloop/common/config/c_log"
	"cicv-data-closedloop/common/util"
	"net/rpc"
	"os"
	"runtime"
	"time"
)

func init() {
	runtime.GOMAXPROCS(1)
	// 初始化日志配置
	c_log.InitLog(commonConfig.LogDir, commonConfig.ControlLogPrefix)
	// 初始化本地配置文件（第1处配置，在本地文件）
	commonConfig.InitLocalConfig()
	// 初始化Oss连接信息
	commonConfig.InitOssConfig()
	// 初始化业务逻辑配置信息，配置文件在oss上（第2处配置，在oss文件）
	commonConfig.InitCloudConfig()
	// 初始化平台任务配置
	commonConfig.InitPlatformConfig()
}

func main() {
	if commonConfig.LocalConfig.Internet {
		RunWithInternet()
	} else {
		RunWithoutInternet()
	}
}

func RunWithInternet() {
	wait := false
	//  轮询任务接口, 决定数采程序启停
	for {
		if wait { // 第一次就不等待一分钟了
			time.Sleep(time.Duration(60) * time.Second)
		}
		wait = true

		// 1. 获取当前设备的任务配置
		task, err := commonConfig.GetConfig()
		if err != nil {
			c_log.GlobalLogger.Error("获取今日任务信息失败：", err)
			continue
		}

		// 2 当前时间需要执行此任务
		// 2.1 判断此任务是否正在执行
		if commonConfig.CurrTask.TaskConfigId == task.TaskConfigId { // 此任务正在执行，退出循环
			continue
		}
		// 此任务没有在执行，需要启动此任务
		// 2.2 判断是否有其他任务正在执行
		if commonConfig.CurrTask.TaskConfigId != "" { // 存在其他任务正在执行，杀死旧的任务
			c_log.GlobalLogger.Info("需要执行的任务id为", task.TaskConfigId, ", 当前任务id为：", commonConfig.CurrTask.TaskConfigId, ", 需要杀死该任务。")
			// 杀死旧的任务
			var killArgs commonService.KillSignal
			// todo 吉利新平台没有DropUploadData字段（旧平台，已采数据：保留/丢弃）
			killArgs = commonService.KillSignal{DropUploadData: true, Restart: false}
			c_log.GlobalLogger.Info("杀死任务，发送rpc结束信号到本地：", killArgs)
			KillRpcClient, err := rpc.Dial("tcp", commonConfig.LocalConfig.Node.Ip+":"+commonConfig.CloudConfig.RpcPort)
			if err != nil {
				// 此处如果连接失败说明采集程序已经停止了
				commonConfig.CurrTask = commonConfig.PlatformConfigStruct{}
				c_log.GlobalLogger.Error("采集程序已经停止：", err)
				continue
			}
			reply := 0
			if err = KillRpcClient.Call("KillService.Kill", killArgs, &reply); err != nil {
				c_log.GlobalLogger.Error("发送rpc请求到master失败：", err)
				// 这里可能会报错 unexpected EOF 但是不影响，先注释 close 和 continue
				//KillRpcClient.Close()
				//continue
			}
			c_log.GlobalLogger.Info("结束任务后，将数据闭环平台配置置空。")
			commonConfig.CurrTask = commonConfig.PlatformConfigStruct{}
			if err = KillRpcClient.Close(); err != nil {
				// 不做处理
			}
		}

		// 开启新的任务
		startMasterOrSlave()
		commonConfig.CurrTask = task
	}
}

func RunWithoutInternet() {
	startMasterOrSlave()
}

func startMasterOrSlave() {
	if _, err := util.ExecuteWithPath(commonConfig.LocalConfig.RestartCmd.Dir, commonConfig.LocalConfig.RestartCmd.Name, commonConfig.LocalConfig.RestartCmd.Args...); err != nil {
		c_log.GlobalLogger.Info("启动新程序失败,【path】=", commonConfig.LocalConfig.RestartCmd.Dir, "【cmd】=", commonConfig.LocalConfig.RestartCmd.Name, commonConfig.LocalConfig.RestartCmd.Args, "：", err)
		os.Exit(-1)
	}
	c_log.GlobalLogger.Info("启动任务，本地执行启动命令：【path】=", commonConfig.LocalConfig.RestartCmd.Dir, "【cmd】=", commonConfig.LocalConfig.RestartCmd.Name, commonConfig.LocalConfig.RestartCmd.Args)
}
